-- libFoxFrameCreator.lua
-- Handles Creating Basic Frames.

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

----------------------------------------------------------------------------------------------------
-- Addon Variables
----------------------------------------------------------------------------------------------------
local LibraryName 		= "FoxFrameCreator";
local LibraryVersion	= "1.0";
local lib;
local Console;

----------------------------------------------------------------------------------------------------
--  Get Library Info
----------------------------------------------------------------------------------------------------
if (LibStub:GetLibrary(LibraryName, true) == nil) then
	lib = LibStub:NewLibrary(LibraryName, LibraryVersion);
else
	lib = LibStub:GetLibrary(LibraryName);
end

----------------------------------------------------------------------------------------------------
--  Get Console
----------------------------------------------------------------------------------------------------
Console 			= LibStub:GetLibrary("FoxConsole");

--================================================================================================--
--=========================================== Header =============================================--
--================================================================================================--

local defaultAnchorInfo = {
	point 			= "Center";
	relativeFrame 	= UIParent;
	relativePoint 	= "Center";
	relativeX		= 0;
	relativeY		= 0;
}
local function GetBorderBackdrop()
	local backdrop			= {};
	backdrop.bgFile			= "Interface\\Tooltips\\UI-Tooltip-Background";
	backdrop.edgeFile		= "Interface\\Tooltips\\UI-Tooltip-Border";
	backdrop.tile			= false;
	backdrop.tileSize		= 16;
	backdrop.edgeSize		= 12;
	backdrop.insets			= {};
	backdrop.insets.left	= 3;
	backdrop.insets.right	= 3;
	backdrop.insets.top		= 3;
	backdrop.insets.bottom	= 3;
	return backdrop;
end
local function GetBorderlessBackdrop()
	local backdrop			= {};
	backdrop.bgFile			= "Interface\\Tooltips\\UI-Tooltip-Background";
	backdrop.edgeFile		= "";
	backdrop.tile			= false;
	backdrop.tileSize		= 16;
	backdrop.edgeSize		= 0;
	backdrop.insets			= {};
	backdrop.insets.left	= 0;
	backdrop.insets.right	= 0;
	backdrop.insets.top		= 0;
	backdrop.insets.bottom	= 0;
	return backdrop;
end

-- Create
function lib.New(name, dontUseBorder, width, height, parent, anchor, inherits)
	Console.CondError(LibraryName, (_G[name] ~= nil) , "Frame Name \""..(name or "").."\" already taken, name must be unique.");
	-- Parameters
	width 		= width  	or 200;
	height 		= height 	or 200;
	parent		= parent 	or UIParent;
	anchor		= anchor 	or defaultAnchorInfo;
	inherits 	= inherits 	or "SecureHandlerStateTemplate, SecureHandlerShowHideTemplate";
	if (inherits == "nil") then
		inherits = nil;
	end
	
	-- Creation
	local self = CreateFrame("FRAME", name, parent, inherits);
	self.overlay = CreateFrame("FRAME", name.."OverLay", self, inherits);
	self.overlay:SetPoint("BottomLeft", 0, 0);
	self.overlay:SetPoint("TopRight", 0, 0);
	self.overlay.label1 = lib.CreateLabel(self.overlay.label1, self.overlay, name, "Center", self.overlay, "Center", 0, 0, 100, 100, "Center", "Middle");
	--self.overlay.label1:SetTextColor(0, 0, 0, 1);
	self.overlay:Hide();
	
	-- Setup
	self = lib.Setup(self, dontUseBorder, width, height, parent, anchor, inherits);
	
	-- Return New Frame
	return self;
end

-- Create
function lib.Setup(self, dontUseBorder, width, height, parent, anchor, inherits)
	Console.CondError(LibraryName, (self == nil) , "Frame doesn't exist, frame must be created first.");
	-- Parameters
	width 		= width  	or 200;
	height 		= height 	or 200;
	parent		= parent 	or UIParent;
	anchor		= anchor 	or defaultAnchorInfo;
	inherits 	= inherits 	or "SecureHandlerStateTemplate, SecureHandlerShowHideTemplate";
	if (inherits == "nil") then
		inherits = nil;
	end
	
	-- Set Size
	self:SetWidth(width);
	self:SetHeight(height);
	
	-- Set Point
	if (anchor ~= "nil") then
		self:SetPoint(anchor.point, anchor.relativeFrame, anchor.relativePoint, anchor.relativeX, anchor.relativeY);
	end
	
	-- Set to Background
	self:SetFrameStrata("BACKGROUND");
	
	-- Setup Border & Backdrop
	if (not dontUseBorder) then
		lib.SetupFrameWithBorder(self);
	else
		lib.SetupFrameWithoutBorder(self);
	end
	
	-- Set Color
	self:SetBackdropColor(0.1, 0.1, 0.1, 0.6);
	
	-- Show
	self:Show();
	
	-- Return New Frame
	return self;
end

-- Setup Back Drop
function lib.SetupFrameWithBorder(self)
	if (self == nil) then
		return;
	end
	self:SetBackdrop(GetBorderBackdrop());
	
	-- Set Color
	self:SetBackdropColor(0.1, 0.1, 0.1, 0.6);
end
function lib.SetupFrameWithoutBorder(self)
	if (self == nil) then
		return;
	end
	self:SetBackdrop(GetBorderlessBackdrop());
end

-- Setup Drag
function lib.MakeFrameDraggable(self)
	if (self == nil) then
		return;
	end

	self:SetMovable(true);
	self:EnableMouse(true);
	self:RegisterForDrag("LeftButton");
	self:SetScript("OnDragStart", self.StartMoving);
	self:SetScript("OnDragStop", self.StopMovingOrSizing);
end
function lib.MakeFrameNotDraggable(self)
	if (self == nil) then
		return;
	end
	
	self:SetMovable(false);
	self:EnableMouse(false);
	self:SetScript("OnDragStart", nil);
	self:SetScript("OnDragStop", nil);
end

-- Create Label
function lib.CreateLabel(self, parent, text, point, anchor, anchorPoint, x, y, width, height, justifyH, justifyV)
	if (self == nil) then
		self = parent:CreateFontString();
	end
	anchor = anchor or self:GetParent();
	self:ClearAllPoints();
	self:SetPoint(point, anchor, anchorPoint, x, y);
	self:SetWidth(width);
	self:SetHeight(height);
	-- Set Font
	self:SetFont("Fonts\\FRIZQT__.TTF", 10, "OUTLINE");
	-- Set Justification
	justifyH = justifyH or "RIGHT";
	self:SetJustifyH(justifyH);
	justifyV = justifyV or "TOP";
	self:SetJustifyV(justifyV);
	
	-- Set Shadow
	self:SetShadowOffset(1, -1);
	-- Set Color
	self:SetTextColor(1.00, 1.00, 1.00);

	self:SetText(text);

	return self;
end

function lib.MakeTitle(self, useColor, size)
	size = size or 14;
	self:SetFont("Fonts\\FRIZQT__.TTF", size, "OUTLINE");
	
	-- Set Color
	if (useColor) then
		self:SetTextColor(1.00, 0.80, 0.00);
	else
		self:SetTextColor(1.00, 1.00, 1.00);
	end
end

-- Create Texture
function lib.CreateTexture(self, parent, texture, point, anchor, anchorPoint, x, y, width, height, layer)
	if (self == nil) then
		self = parent:CreateTexture(nil, layer);
	end

	self:ClearAllPoints();
	self:SetPoint(point, anchor, anchorPoint, x, y);
	self:SetWidth(width);
	self:SetHeight(height);
	self:SetTexture(texture);

	return self;
end

-- Create Button
function lib.CreateButton(self, name, parent, texture, point, anchor, anchorPoint, x, y, width, height)
	if (self == nil) then
		if (parent == not nil) then
			name = parent:GetName()..name;
		end
		self = CreateFrame("Button", name, parent);
	else
		self:SetParent(parent);
	end
	
	-- Setup Appearance
	self:SetWidth(width);
	self:SetHeight(height);
	self:SetNormalTexture(texture);
	self:SetPoint(point, anchor, anchorPoint, x, y);
	
	return self;
end